/***************************************************************************
 *  How to use SLCIO files from HepSim, and how to build anti-KT jets 
 *  S.Chekanov (ANL) chekanov@anl.gov
 *  A library for HEP events storage and processing based on Google's PB   
 *  The project web site: http://atlaswww.hep.anl.gov/hepsim/
****************************************************************************/

#include<iostream>
#include<fstream>
#include<stdlib.h>

// check directory
#include <sys/stat.h>
#include <TROOT.h>
#include <TFile.h>
#include <TH1D.h>
#include "TMath.h"
#include "TNtupleD.h"
#include "TLorentzVector.h"


#include"time.h"
#include <dirent.h>
#include <string>
#include <vector>
#include <map>
struct stat sb;

// fastjet
#include "fjcore.hh"

#include "lcio.h"
#include <stdio.h>
#include "IO/LCReader.h"
#include "IMPL/LCTOOLS.h"
#include "EVENT/LCEvent.h"
#include "EVENT/LCRunHeader.h"
#include "EVENT/CalorimeterHit.h"
#include "EVENT/RawCalorimeterHit.h" 
#include "EVENT/ReconstructedParticle.h"
#include "IOIMPL/LCFactory.h"
#include "EVENT/MCParticle.h"
#include "EVENT/LCCollection.h"
#include "IMPL/LCEventImpl.h"
#include "UTIL/LCTOOLS.h"
#include "UTIL/Operators.h"
#include "UTIL/LCIterator.h"
#include "IMPL/LCCollectionVec.h"
#include "IMPL/MCParticleImpl.h"
#include "EVENT/MCParticle.h"
#include "EVENT/ReconstructedParticle.h"
#include "EVENT/Track.h"
#include "EVENT/Cluster.h"
#include "EVENT/CalorimeterHit.h"
#include "EVENT/SimCalorimeterHit.h"

using namespace std;

// find all files inside a directory
std::vector<std::string> open(std::string path = ".") {
  std::vector<std::string> files;
  if (stat(path.c_str(), &sb) == 0 && S_ISDIR(sb.st_mode)) { 
    DIR*    dir;
    dirent* pdir;
    dir = opendir(path.c_str());
    while (pdir = readdir(dir)) {
        string rfile=path + "/" + pdir->d_name;
        if (rfile.find(".slcio") != std::string::npos) 
             files.push_back(rfile); 
    }
    } else {
        cout << "Directory " << path << " does not exist!!" << endl;
        std::exit(0); 
        }
    return files;
}


// main example
int main(int argc, char **argv)
{



  string dir="./data/";
  std::vector<std::string> files = open(dir);


  string outputfile="output.root";
  cout << "\n -> Output file is =" << outputfile << endl;
  TFile * RootFile = new TFile(outputfile.c_str(), "RECREATE", "Histogram file");
  TH1D * h_pt = new TH1D("jet_pt", "pt",100,0,1000);
  TH1D * h_eta = new TH1D("jet_eta", "eta", 40, -10, 10);
  const char* tupleNames = "px:py:pz:vx:vy:vz:ex:ey:ez:pdg:np:nd:gs:ss" ;
  TNtupleD *tuple= new TNtupleD("MCParticle", "", tupleNames );
  

  // fastjet
  double R = 0.5;
  fjcore::JetDefinition jet_def(fjcore::antikt_algorithm, R);
  //JetDefinition jet_def(fastjet::kt_algorithm, Rparam, strategy);
  //JetDefinition jet_def(fastjet::cambridge_algorithm, Rparam, strategy);
  //JetDefinition jet_def(fastjet::antikt_algorithm, Rparam, strategy);


  double px,py,pz,e,vx,vy,vz,ex,ey,ez ;
  int pdg,np,nd,gs,ss ;

   int nEvents = 0  ;

  // loop over all files
  for(unsigned int m=0; m < files.size(); m++){
  string Rfile=files[m];

  IO::LCReader* lcReader = IOIMPL::LCFactory::getInstance()->createLCReader() ;
  lcReader->open( Rfile.c_str() ) ;

  EVENT::LCEvent* evt=0 ;

  
 //----------- the event loop -----------
  while( (evt = lcReader->readNextEvent()) != 0 ) {
    if (nEvents==0) UTIL::LCTOOLS::dumpEvent( evt ) ;
    nEvents ++ ;
    if (nEvents%10==0)
          cout <<  " # Events=" << nEvents << endl;


    std::string mcpName("MCParticle") ;
    // get truth
    IMPL::LCCollectionVec* col = (IMPL::LCCollectionVec*) evt->getCollection( mcpName  ) ;
    int nMCP = col->getNumberOfElements() ;
    bool take=false;
    for(int i=0 ; i<nMCP ; ++i){
      EVENT::MCParticle* mcp =  (EVENT::MCParticle*) col->getElementAt(i) ;
      px = mcp->getMomentum()[0] ;
      py = mcp->getMomentum()[1] ;
      pz = mcp->getMomentum()[2] ;
      e = mcp->getMomentum()[3] ;

      TLorentzVector p;
      p.SetPxPyPzE(px,py,pz,e);
      //if (abs(p.PseudoRapidity()) < 1.5 && p.Perp()>1) take=true; 
      pdg = mcp->getPDG() ;
      np = mcp->getParents().size() ;
      nd = mcp->getDaughters().size() ;
      gs = mcp->getGeneratorStatus() ;
      ss = mcp->getSimulatorStatus() ;
      tuple->Fill(px,py,pz,vx,vy,vz,ex,ey,ez,pdg,np,nd,gs,ss) ;
  }


   // take event with central production
   //if (take==false) continue;


  // calorimeters
  // look up at: lcio/build/include/EVENT
  double xsum=0;
  IMPL::LCCollectionVec* col2 = (IMPL::LCCollectionVec*) evt->getCollection( "PandoraPFOCollection"  ) ;
  int nPFO = col2->getNumberOfElements() ;
    for(int i=0 ; i<nPFO ; ++i){
      EVENT::ReconstructedParticle* mcp =  (EVENT::ReconstructedParticle*) col2->getElementAt(i) ;
      double px = mcp->getMomentum()[0];
      double py = mcp->getMomentum()[1];
      double pz = mcp->getMomentum()[2];
      double e = mcp->getMomentum()[3];
      xsum=xsum+e;
  }
  cout << "PFA=" << xsum << endl;

  //tracks 
  IMPL::LCCollectionVec* col3 = (IMPL::LCCollectionVec*) evt->getCollection( "Tracks"  ) ;
  int nTRK = col3->getNumberOfElements() ;
    for(int i=0 ; i<nTRK ; ++i){
      EVENT::Track* mcp =  (EVENT::Track*) col3->getElementAt(i) ;
      //px = mcp->getMomentum()[0] ;
      //py = mcp->getMomentum()[1] ;
      //pz = mcp->getMomentum()[2] ;
  }



  // clusters
  xsum=0;
  IMPL::LCCollectionVec* col5 = (IMPL::LCCollectionVec*) evt->getCollection( "ReconClusters"  ) ;
  int nCL = col5->getNumberOfElements() ;
    for(int i=0 ; i<nCL ; ++i){
      EVENT::Cluster* mcp =  (EVENT::Cluster*) col5->getElementAt(i) ;
      double e = mcp->getEnergy();
      xsum=xsum+e;
  }
  cout << "Cal energy=" << xsum << endl;



//  IMPL::LCCollectionVec* col4 = (IMPL::LCCollectionVec*) evt->getCollection( "HcalBarrelHits"  ) ;
  IMPL::LCCollectionVec* col4 = (IMPL::LCCollectionVec*) evt->getCollection( "HAD_BARREL"  ) ;

  int nBCalHit = col4->getNumberOfElements() ;

    xsum=0;
    for(int i=0 ; i<nBCalHit ; ++i){
     EVENT::CalorimeterHit* mcp1 =  (EVENT::CalorimeterHit*) col4->getElementAt(i) ;
     //EVENT::SimCalorimeterHit* mcp =  (EVENT::SimCalorimeterHit*) col4->getElementAt(i) ;
      EVENT::CalorimeterHit* mcp =  (EVENT::CalorimeterHit*) col4->getElementAt(i) ; 
      float e = mcp->getEnergy();
      xsum=xsum+e;

     // int   type=mcp-> getType();
      int   id0=mcp->getCellID0();
      int   id1=mcp->getCellID1(); 

      //double ee=mcp->getCorrectedEnergy() ;
      ///position of the hit in world coordinates
      float x = mcp->getPosition()[0] ;
      float y = mcp->getPosition()[1] ;
      float z = mcp->getPosition()[2] ;
  }

   cout << "Hit sum=" << xsum << endl;
  }



  lcReader->close() ;
  delete lcReader ;

   } // close the file 

   //m_ntuple->Fill();
   RootFile->Write();
   RootFile->Print();
   RootFile->Close();

   cout << "Writing ROOT file "+ outputfile << endl;

    return 0;
}
